from time import sleep
import RPi.GPIO as GPIO
from threading import Thread


class ServoKontrol:

    def __init__(self, pin, GPIOSetup=GPIO.BOARD):
        GPIO.setmode(GPIOSetup)

        GPIO.setup(pin, GPIO.OUT)
        self.pwm = GPIO.PWM(pin, 50)
        self.pwm.start(0)
        self.pin = pin
        self.hedefAci = 90
        self.suankiAci = 90
        self.uyuduMu = True
        self.surekliMi = False

    def surekliDonmeyeAyarla(self):
        self.surekliMi = True
        GPIO.output(self.pin, True)

    def tekAciDonmeyeAyarla(self):
        self.surekliMi = False
        GPIO.output(self.pin, False)

    def __tekAciAyarlaAsil__(self):
        sinyalUzunlugu = self.hedefAci / 18 + 2
        GPIO.output(self.pin, True)
        self.pwm.ChangeDutyCycle(sinyalUzunlugu)

        deltaAci = abs(self.hedefAci - self.suankiAci)
        gerekenUyku = deltaAci / 150
        sleep(gerekenUyku)  # experimental value
        GPIO.output(self.pin, False)
        self.pwm.ChangeDutyCycle(0)
        self.suankiAci = self.hedefAci
        self.uyuduMu = True

    def __surekliAciAyarlaAsil__(self, aci):
        duty = aci / 18 + 2
        self.pwm.ChangeDutyCycle(duty)

    def aciAyarla(self, aci):

        self.hedefAci = aci

        if self.surekliMi:
            self.__surekliAciAyarlaAsil__(self.hedefAci)
        elif self.uyuduMu and (self.suankiAci is not self.hedefAci):
            self.uyuduMu = False





from .KumandaKontrol import Kumanda
from .MotorKontrol import MotorKontrol
from .ServoKontrol import ServoKontrol
from .UltrasonikSensor import UltrasonikSensor
from .PiKamera20 import HizlandirilmisPiKamera



from picamera import PiCamera
from picamera.array import PiRGBArray
from threading import Thread
import cv2

class HizlandirilmisPiKamera:

    def __init__(self, cozunurluk=(640, 480)):

        self.camera = PiCamera()
        self.camera.resolution = cozunurluk
        self.hamKare = PiRGBArray(self.camera, size=self.camera.resolution)
        self.yayin = self.camera.capture_continuous(self.hamKare, format="bgr", use_video_port=True)
        self.suAnkiKare = None

        self.penceredeGosterilecekler = dict()
        self.kameraGostermeAktif = False

    def veriOkumayaBasla(self):

        Thread(target=self.__veriGuncelle__, args=()).start()
        return self

    def __veriGuncelle__(self):

        for f in self.yayin:

            self.suAnkiKare = f.array
            self.hamKare.truncate(0)

    def veriOku(self):

        return self.suAnkiKare

    def kareyiGoster(self, pencereninIsmi="frame", gosterilecekGoruntu=None):
        if gosterilecekGoruntu is None:
            self.penceredeGosterilecekler[pencereninIsmi] = self.suAnkiKare
        else:
            self.penceredeGosterilecekler[pencereninIsmi] = gosterilecekGoruntu

        if not self.kameraGostermeAktif:
            Thread(target=self.__kareyiGostermeyiGuncelle__, args=()).start()

    def __kareyiGostermeyiGuncelle__(self):

        self.kameraGostermeAktif = True

        while True:

            for isim in self.penceredeGosterilecekler.copy():
                cv2.imshow(isim, self.penceredeGosterilecekler[isim])

            key = cv2.waitKey(1)

            if key == ord("q"):
                cv2.destroyAllWindows()
                break


from pololu_drv8835_rpi import motors
import math


class MotorKontrol:

    def __init__(self):
        self.hizSag = 0
        self.hizSol = 0

    def hizlariAyarla(self, hizSag, hizSol):
        self.hizSag = hizSag
        self.hizSol = hizSol

        480 if hizSag > 480 else hizSag
        -480 if hizSag < -480 else hizSag

        480 if hizSol > 480 else hizSol
        -480 if hizSol < -480 else hizSol

        motors.setSpeeds(hizSag, hizSol)

    def kumandaVerisiniMotorVerilerineCevirme(self, x, y):
        r = math.hypot(x, y)
        t = math.atan2(y, x)

        # rotate by 45 degrees
        t += math.pi / 4

        # back to cartesian
        left = r * math.cos(t)
        right = r * math.sin(t)

        # rescale the new coords
        left = left * math.sqrt(2)
        right = right * math.sqrt(2)

        # clamp to -1/+1
        left = max(-1, min(left, 1))
        right = max(-1, min(right, 1))

        return int(left * 480), -int(right * 480)



import pygame
from threading import Thread


class Kumanda:

    def __init__(self):

        pygame.init()

        pygame.joystick.init()

        self.j = pygame.joystick.Joystick(0)
        self.j.init()

        self.butonlar = []
        self.solX = 0
        self.solY = 0
        self.sagX = 0
        self.sagY = 0

    def dinlemeyeBasla(self):

        Thread(target=self.__yenile__, args=()).start()
        return self

    def __yenile__(self):

        while True:
            for e in pygame.event.get():
                if (e.type == pygame.JOYBUTTONDOWN and e.button not in self.butonlar):
                    self.butonlar.append(e.button)
                if (e.type == pygame.JOYBUTTONUP and e.button in self.butonlar):
                    self.butonlar.remove(e.button)
                if (e.type == pygame.JOYAXISMOTION):
                    if (e.axis == 0):
                        self.solX = e.value
                    elif (e.axis == 1):
                        self.solY = e.value
                    elif (e.axis == 2):
                        self.sagX = e.value
                    elif (e.axis == 3):
                        self.sagY = e.value

    def solVerileriOku(self):
        return self.solX, self.solY

    def sagVerileriOku(self):
        return self.sagX, self.sagY

    def butonlariOku(self):
        return self.butonlar

    def verileriOku(self):
        return self.solVerileriOku(), self.sagVerileriOku(), self.butonlariOku()



import wiringpi

# Motor speeds for this library are specified as numbers
# between -MAX_SPEED and MAX_SPEED, inclusive.
_max_speed = 480  # 19.2 MHz / 2 / 480 = 20 kHz
MAX_SPEED = _max_speed

io_initialized = False
def io_init():
  global io_initialized
  if io_initialized:
    return

  wiringpi.wiringPiSetupGpio()
  wiringpi.pinMode(12, wiringpi.GPIO.PWM_OUTPUT)
  wiringpi.pinMode(13, wiringpi.GPIO.PWM_OUTPUT)

  wiringpi.pwmSetMode(wiringpi.GPIO.PWM_MODE_MS)
  wiringpi.pwmSetRange(MAX_SPEED)
  wiringpi.pwmSetClock(2)

  wiringpi.pinMode(5, wiringpi.GPIO.OUTPUT)
  wiringpi.pinMode(6, wiringpi.GPIO.OUTPUT)

  io_initialized = True

class Motor(object):
    MAX_SPEED = _max_speed

    def __init__(self, pwm_pin, dir_pin):
        self.pwm_pin = pwm_pin
        self.dir_pin = dir_pin

    def setSpeed(self, speed):
        if speed < 0:
            speed = -speed
            dir_value = 1
        else:
            dir_value = 0

        if speed > MAX_SPEED:
            speed = MAX_SPEED

        io_init()
        wiringpi.digitalWrite(self.dir_pin, dir_value)
        wiringpi.pwmWrite(self.pwm_pin, speed)

class Motors(object):
    MAX_SPEED = _max_speed

    def __init__(self):
        self.motor1 = Motor(12, 5)
        self.motor2 = Motor(13, 6)

    def setSpeeds(self, m1_speed, m2_speed):
        self.motor1.setSpeed(m1_speed)
        self.motor2.setSpeed(m2_speed)

motors = Motors()



import RPi.GPIO as GPIO
from time import sleep, time
from threading import Thread


class UltrasonikSensor:

    def __init__(self, echo, trig, setup=GPIO.BOARD):
        self.echo = echo
        self.trig = trig

        self.sure = 0

        self.anlikOlcum = 0

        GPIO.setmode(setup)

        GPIO.setup(self.trig, GPIO.OUT)
        GPIO.setup(self.echo, GPIO.IN)

        GPIO.output(trig, False)

    def mesafeOlcmeyeBasla(self):

        Thread(target=self.__mesafeOlc__).start()
        sleep(0.2)

    def mesafeOku(self):
        return self.anlikOlcum

    def __mesafeOlc__(self):

        while True:
            GPIO.output(self.trig, True)
            sleep(0.0001)
            GPIO.output(self.trig, False)

            sinyal_baslangic = time()

            while GPIO.input(self.echo) == 1:
                sinyal_bitis = time()

                self.sure = sinyal_bitis - sinyal_baslangic
                self.anlikOlcum = self.sure * 17150


from setuptools import setup
import subprocess
import os

os.system("echo hello")
os.system('sudo apt-get install libcblas-dev')
os.system('sudo apt-get install libhdf5-dev')
os.system('sudo apt-get install libhdf5-serial-dev')
os.system('sudo apt-get install libqtwebkit4')
os.system('sudo apt-get install libqt4-test')
os.system('sudo apt-get install libatlas-base-dev')
os.system('sudo apt-get install libjasper-dev')

setup(
    name = "Pi20",
    version = "0.1.6",
    author = "Andy Emre Kocak, Rana Taki, Yoel Kastro, Yasar Idikut",
    author_email = "emre.kocak@hisarschool.k12.tr, rana.taki@hisarschool.k12.tr, sarp.kastro@hisarschool.k12.tr, yasar.idikut@hisarschool.k12.tr",
    description = "Library that makes use of sensors, motors, and servos in the PiWars Turkey robot kit by HisarCS",
    packages = ["Pi20"],
    classifiers=["Development Status :: 4 - Beta"],
    install_requires=[
        'picamera',
        'pygame',
        'RPi.GPIO',
        'wiringpi',
        'numpy',
        'opencv-contrib-python==3.4.3.18'
    ]

)
